#!/usr/bin/perl -w
#
# Name ............: simplebackup.pl
# Type ............: Perl Script
#
# Version .........: 1.2.0
# Last Update .....: 13 May 2004
#
# License .........: GNU GENERAL PUBLIC LICENSE Version 2, June 1991.
#
# Authors ..........: Miguel Angelo Martins Leite,
# Script E-Mail ....: miguel.angelo.public@migas.mine.nu
# Site .............: http://migas.mine.nu/index.php?pag=en.myapps&subpag=simplebackup
#
# Objective .......: Read a configuration file and do a full backup of a directory
#                    into a compress file, the compress file will be placed on a chosen
#                    directory, or on a remote ftp server.
#                    It also supports backup session management, for multiple backup
#                    sessions, for example in order to keep the backups of a entire week.
#
# Depends on ......: Perl (v5.6 or over)
#                    Following archive programs (executables must be on your $PATH variable)
#                      * rar/winrar (v3.0 or over) : for rar backup type
#                      * info-zip (v2.3 or over) : for zip backup type
#                      * tar : for tar, tar compress, tar gzip, tar bzip2 backup types
#                      * compress : for tar compress backup type
#                      * gzip : for tar gzip backup type
#                      * bzip2 : for tar bzip2 backup type
#
# Runs On .........: Windows NT4, Windows 2000, Windows XP and  All Unix systems
#
#
# Revision History :
#                    1.0.0 RC1 - 10 March 2004 - Miguel Angelo Martins Leite
#                      1. First version.
#
#                    1.0.0 RC2 - 13 March 2004 - Miguel Angelo Martins Leite
#                      1. Loads of tests on Unix (Linux) and Windows (XP) systems, most with
#                         success.
#                      2. Small fixes on Unix systems (Linux).
#                      3. Added support to run scripts/batch files/commands before and
#                         after a backup. Can be used to for example to shut-down a database
#                         do the backup, and in the end bring the database backup on-line.
#                      4. Improved the log messages.
#                      5. Improved a lot the documentation (readme.txt).
#                      6. Added support for zip backup files, using the info-zip software
#                         (http://www.info-zip.org/pub/infozip/) and included a Windows
#                         version to the simplebackup package, this will help Windows users
#                         get the backups running faster.
#
#                    1.0.0 RC3 - 17 March 2004 - Miguel Angelo Martins Leite
#                      1. Continuing testes on Unix (Linux) and Windows (XP) systems.
#                      2. Small fixes on the documentation (readme.txt).
#                      3. Fixed a bug that reported a success backup even if the backup failed.
#                         It happened when the write move of the backup file into the final
#                         backup directory failed, fallowed by a success backup reported.
#                         Now if the move of the file fails (lack of space, permissions, etc)
#                         the backup is reported has failed.
#                      4. Added a new field on the log file message, the configuration file name,
#                         this should help people that only use a single log file for all backups.
#                         Thanks to my friend Alexandre Pereira for this suggestion.
#                      5. Added a new option on the configuration file ( backup_compression_ratio )
#                         which enables users to select the compression ratio that they want to use
#                         on their compression files. Options are generic for all archive formats, 
#                         none ; minimal ; average and maximal.
#                         Notice that not all formats support the compression ratio in that case
#                         the option is adapted or ignored, and simplebackup will not fail.
#                         Thanks to my friend Alexandre Pereira for this suggestion.
#
#                    1.0.0 RC4 - 19 March 2004 - Miguel Angelo Martins Leite
#                      1. Continuing testes on Unix (Linux) and Windows (XP) systems.
#                      2. Small fixes on the documentation (readme.txt).
#
#                    1.0.0 RC5 - 21 March 2004 - Miguel Angelo Martins Leite
#                      1. Continuing testes on Unix (Linux) and Windows (XP) systems.
#                      2. Improved the documentation (readme.txt).
#                      3. Fixed a small message that was miss representing the operation, it was
#                         reporting "Executing before backup procedure using command...", when it
#                         should be reporting "Executing after backup procedure using command..."
#                      4. Improved a bit more the log file, reporting the path him self.
#                      5. Added a new argument (--tconf) and function show_config(), --tconf tests
#                         the a given configuration file (example: simplebackup.pl --tconf file.conf),
#                         and the show_config() function will print the file configuration.
#                         And the most important reason, now i can use this to produce a screenshot
#                         to place on the program site hehehehe :).
#                      6. Centralized the operating system null device on the configuration hash
#                         (%global_config), and this value is now loaded on the read_config() function.
#                      7. Changed the simplebackup.pl intro screen to match the changes.
#
#                    1.0.0 RC6 - 27 March 2004 - Miguel Angelo Martins Leite
#                      1. Continuing testes on Unix (Linux) and Windows (XP) systems.
#                      2. Improved the documentation (readme.txt).
#                      3. Small improvement on the log messages.
#                      4. Changed the Windows NT4 [ host_os ] parameter from nt4 into winnt4, it
#                         makes more sense this way, i think.
#                      5. Small fix on Windows NT4, simplebackup was letting Windows move command
#                         print a message when it shouldn't.
#                      6. Added a new warning, if simplebackup is unable to write into the backup
#                         log file, it will complain generally A LOT !, but will not fail.
#                      7. Fixed the zip support on Unix, info-zip does not support the -S parameter
#                         on Unix systems, -S is only for Microsoft Windows Systems, and Mac's.
#                      8. Remove code that added the null device into the command defined by
#                         "run_before_backup" and the "run_after_backup" on the configuration file,
#                         this might cause problems on some Windows programs that simply do not
#                         understand the concept of text redirect "command 2>nul", one example of
#                         this is Microsoft Outlook XP.
#                         If you want to redirect a output of a external command into the null
#                         device you must add that option into the "run_before_arguments" and the
#                         "run_afters_arguments" in the configuration file.
#                      9. Added support for directories and file names with spaces, example
#                         [ d:\test\name 1 with spaces\name 2 with spaces\final directory with spaces ]
#                         or [ /name 1 with spaces/name 2 with spaces/final directory with spaces ].
#
#                    1.0.0 28 March 2004 - Miguel Angelo Martins Leite
#                      1. Final stable version, simplebackup was tested on several operating systems
#                         and several situations. This is a copy of RC6.
#
#                    1.1.0 RC1 - 12 April 2004 - Miguel Angelo Martins Leite
#                      1. Fixed a bug, in the move_file() function, that in some situations could
#                         make it fail to report a missed moved file. In the previous simplebackup
#                         version the bug existed but failed to "work" hehehehe, but because of
#                         changes on the function the bug "worked" ;)
#                      2. Added support for backup's into FTP servers, with url check up, support
#                         for FTP passive mode and open FTP proxies.
#                      3. Added new options to the sample configuration file (simplebackup_example.conf).
#                      4. Added the function local_db_file_history() that manages a simple db file,
#                         that contains the backup file list managed by simplebackup. This is done
#                         to permit simplebackup to operate with "strange" ftp servers that produce
#                         weird file lists (ls/dir commands).
#                      5. Added support to show the FTP details on the --tconf option (test
#                         configuration file).
#                      6. Improvements on the program commentaries.
#                      7. Changed a bit the log messages.
#                      8. Previous backup configuration files (from simplebackup 1.0.0) remain 100%
#                         compatible.
#
#                    1.1.0 RC2 - 13 April 2004 - Miguel Angelo Martins Leite
#                      1. Continuing testes on Unix (Linux) and Windows (XP) systems.
#                      2. Improved the documentation (readme.txt).
#                      3. General code change in order to simplify and improve the program performance
#                         in the function read_config().
#                      4. Added extra security to the move_file() function, in order to prevent
#                         writing over already existing files.
#                      5. Small improvement on the log messages.
#                      6. Changed a bit the simplebackup.pl intro screen to match the changes.
#
#                    1.1.0 RC3 - 16 April 2004 - Miguel Angelo Martins Leite
#                      1. Continuing testes on Unix (Linux) and Windows (XP) systems.
#                      2. Improved the documentation (readme.txt).
#                      3. Bug fix on the session management, it was incorrectly reducing one backup
#                         session.
#                      4. Bug fix on a situation related to erasing data from the database file, if
#                         simplebackup was using the database file and working in "local file system"
#                         mode.
#                      5. Bug fix on a problem that prevented backup's into the / (root) path of a ftp
#                         server.
#                      6. To make it more coherent with the log_file name, changed the name of variables
#                         from local_db_file_history into local_db_history_file, and to prevent
#                         confusions also change the name of the function local_db_file_history() into
#                         work_local_db_history_file().
#                      7. Improvements on the program commentaries.
#                      8. Changed a bit the log messages.
#
#                    1.1.0 RC4 - 29 April 2004 - Miguel Angelo Martins Leite
#                      1. Added a new security mechanism on the ftp backups. If the upload of the backup
#                         file fails simplebackup will attempt to erase the partially upload file, since
#                         it is damaged.
#                         Plus in situations where simplebackup is using a local database file, there for
#                         using the function local_db_history_file(), if the erase of the partially uploaded
#                         uploaded file fails the file name will be added into the database file. This is
#                         done in order to prevent constant backup failures, backup's will recover more
#                         easily.
#                       2. Fixed the sample configuration file for windows.
#
#                    1.1.0 - 01 May 2004 - Miguel Angelo Martins Leite
#                      1. New stable version of simplebackup, this is a copy of RC4.
#                      2. Main features of this new version compared to version 1.0.0 are:
#                         . Support backup's into FTP servers,
#                         . Possibility to use a local database file to manage backup session,
#                         . Improved program commentaries,
#                         . Improved backup log,
#                         . Previous backup configuration files from version 1.0.0 remain 100%
#                           compatible.
#
#                    1.2.0 RC1 - 11 May 2004 - Miguel Angelo Martins Leite
#                      1. Added support for backup mail reporting, using SMTP mail servers only,
#                         simplebackup can now report the backup results into one or more e-mail
#                         addresses, this means small changes all over the program. Specially
#                         into the functions read_config() and write_file_log().
#                      2. Added new options to the sample configuration file (simplebackup_example.conf).
#                      3. Added support to show the mail details on the --tconf option (test
#                         configuration file).
#                      4. Added the function send_email() that actualy attempts to send the backup
#                         mail report.
#                      5  Added the function get_mail_date() that returns the system date in mail format,
#                         this function is used by the send_email() function.
#                      6. For security reasons the code that wrote the ftp password into the log
#                         file was removed.
#                      7. Changed the default ftp timeout value from 240 to 480, this is a attempt to make
#                         simplebackup even more tolerant to network failures.
#                      8. The mail server timeout value is also 480.
#                      9. Now simplebackup will report the hostname where it's running in the log file and
#                         in the e-mail subject.
#                     10. Bug fix on the readme.txt file, my friend is called Nuno Muoz and not Nuno Munhoz.
#                     11. Previous backup configuration files (from simplebackup 1.0.0 and 1.1.0)
#                         remain 100% compatible.
#
#                    1.2.0 RC2 - 13 May 2004 - Miguel Angelo Martins Leite
#                      1. Continuing testes on Unix (Linux) and Windows (XP) systems.
#                      2. Improved the documentation (readme.txt).
#                      3. Small improvement on the log messages.
#                      4. Improved a bit the backup report e-mail.
#                      5. Fixed the english on the sample configuration file.
#                      6. Bug fix on the e-mail subject field, simplebackup was not correctly reporting the
#                         configuration file name, it was reporting teste.conf over and over again.
#
#
# Future Plans ....:
#                   Adding support scp (secure copy protocol) upload,  in order to backup into a
#                   scp server.
#
#

##########################
# Modules
##########################
use strict; # implement a more strict language
use Net::FTP; # support ftp protocol
use Net::SMTP; # support smtp protocol
use Sys::Hostname; # to enable the possibility to get the machine hostname
use Fcntl qw(:DEFAULT :flock);  # file locking for the logs


##########################
# Global Variables
##########################
my $script_version = "1.2.0";
my $script_last_update = "13 May 2004";
my $script_license = "GNU GENERAL PUBLIC LICENSE Version 2, June 1991.";
my $script_authors = "Miguel Angelo Martins Leite";
my $script_email = "miguel.angelo.public\@migas.mine.nu";
my $script_site = "http://migas.mine.nu/index.php?pag=en.myapps&subpag=simplebackup";
my %global_config = (    # script configuration hash
    "configuration_file"        => "",    # Path into the configuration file, ex: /some/path/a_file.conf
    "configuration_file_name"   => "",    # Configuration file name, ex: a_file.conf
    "host_os"                   => "",    # Operating system running the script
    "host_null_device"          => "",    # Operating system null device
    "backup_type"               => "",    # Backup type: rar file, tar.bz2 file, tar.gz file, tar
    "backup_compression_ratio"  => "",    # Chose backup compression ratio:
                                          #   none = no compression = 0 (rar, info-zip) or 1 (gzip,
                                          #   bzip2)
                                          #   minimal = minimal compression = 1
                                          #   average = average type of compression = 3 (rar) or
                                          #             6 (info-zip,gzip,bzip2)
                                          #   maximal = maximal compression = 5 (rar) or
                                          #   9 (info-zip, gzip, bzip2)
                                          # Note tar and compress formats are not affected by the
                                          # argument
    "backup_real_ratio"         => "",    # The actual number to be passed into the compressor,
                                          # 0 up to 5 or 9
    "backup_mode"               => "",    # Where is the backup file is going, configured on the
                                          # read_config() function.
                                          #  local file system = for local file system directory, this
                                          #                      also included windows mounted drives
                                          #                      and unix mounted directories (devices,
                                          #                      nfs, samba, etc).
                                          #                ftp = if the backup is going into a remote
                                          #                      ftp server.
                                          #                scp = if the backup is going into a remote
                                          #                      server using.
                                          #           scp (secure copy)
    "ftp_server"                => "",    # Remote ftp server.
    "ftp_server_port"           => "",    # Remote ftp server port number.
    "ftp_username"              => "",    # FTP Username.
    "ftp_password"              => "",    # FTP Password.
    "ftp_directory"             => "",    # FTP directory where we will upload the backup file.
    "ftp_passive_mode"          => "",    # FTP Passive mode [ 0 ; 1 ].
    "ftp_passive_mode_real"     => "",    # The real parameter passed into the Net:FTP, here is
                                          # only [ 0 ; 1 ].
    "ftp_proxy"                 => "",    # FTP Proxy, must work in open mode.
    "ftp_timeout"               => "",    # Ftp timeout
    "mail_type"                 => "",    # The backup server, empty for no e-mail reporting, smtp for smtp mail server type
                                          # other backup server types (example imap) are not yet supported
    "mail_server"               => "",    # Mail server name/address.
    "mail_port"                 => "",    # Mail server port (default is 25)
    "mail_timeout"              => "",    # Mail server timeout
    "mail_username"             => "",    # Mail username (might be required to smtp login).
    "mail_password"             => "",    # Mail password, if it's not empty simplebackup will attempt
                                          # to login into the mail server.
    "mail_from_address"         => "",    # E-Mail address from,    where the report comes from.
    "mail_to_addresses"         => "",    # E-Mail address(es) to,  where the report is/are going.
    "input_backup_dir"          => "",    # The directory to backup
    "output_backup_dir"         => "",    # Where the final compress file will be placed
    "temporary_dir"             => "",    # Where the data shall be compress, this is a temporary
                                          # destiny for the compressed file
    "keep_last_n_files"         => "",    # Number of the last backup files that shall be keep
                                          # 5 for example the script shall erase all compress backup
                                          # file except the last 5 sessions + the current backup
                                          # session
                                          # Notes : -1  -> all backup files shall be keep
                                          #          0  -> only the present backup file shall be keep
    "run_before_backup"         => "",    # Optional script/batch file/command to run before backup
    "run_before_arguments"      => "",    # Optional arguments for the script/batch file/command defined
                                          # by run_before_backup
    "run_after_backup"          => "",    # Optional script/batch file/command to run after backup
    "run_after_arguments"       => "",    # Optional arguments for the script/batch file/command defined
                                          # by run_after_backup
    "log_file"                  => "",    # Where the log file shall live
    "local_db_history_file"     => "",    # Database file that contains the files managed by
                                          # simplebackup, this USUALLY is not REQUIRED
);


##########################
# Functions
##########################

#
# Show a small command line help
# Will return: 0
# Explain: 0 is sucess
sub show_commands {

    my $my_script_version = $_[0]; # script version parameter
    my $my_script_last_update = $_[1]; # script lastupdate version
    my $my_script_license = $_[2]; # script license
    my $my_script_authors = $_[3]; # script authors
    my $my_script_email = $_[4]; # script email
    my $my_script_site = $_[5]; # script site

    print <<EOF;

  Name .....: simplebackup.pl v$my_script_version ($my_script_last_update)

  License ..: $script_license
  Author ...: $my_script_authors
  E-Mail ...: $my_script_email
  Site .....: $my_script_site

  Objective : Do a simple backup of a directory by compressing the backup
              directory into a archive file and placing it, into some other
              path or machine including ftp servers. The archive file can be
              of one of the following types: rar, zip, tar, tar + compress,
              tar + gzip or tar + bzip2.

  Testing Configuration :
    Unix ...> simplebackup.pl --tconf full/path/config_file.conf
    Windows > perl -w simplebackup.pl --tconf full\\path\\config_file.conf

  Backup Execution :
    Unix ...> simplebackup.pl --conf full/path/config_file.conf
    Windows > perl -w simplebackup.pl --conf full\\path\\config_file.conf

  Notes :
    1 > Don't forget to edit and change the configuration file.
    2 > One configuration file per backup type/backup directory.
    3 > To prevent wierd backup's always define full path's on the
        configuration file.
    4 > It's not advisable to share a single backup directory for all your
        backups, if you want something similar create a main backup
        directory and then several sub-directories, one sub-directory per
        backup type/backup directory.
    5 > You can use this program to create automatic backups using Unix Cron,
        Windows Task Scheduler, etc.

EOF

    return 0; # always return sucess
}


#
# Test the command line parameters/commands
# Will return: 0 ; 1
# Explain: 0 is sucess ; 1 failure
sub test_commands {

    my @parameters = @_; # the prompt parameters
    # scalar counts a array
    if ( scalar(@parameters) != 2 or ($parameters[0] ne "--conf" and $parameters[0] ne "--tconf") ) {
        return 1;
    }
    else { # --conf file.conf > is read config and do backup ; --tcont file.conf is read config and show is values
        return 0;
    }
}


#
# Read (and test) the configuration file into the global var %GLOBAL_CONFIG
# Will return: 0 ; 1 ; 2 ; 3 ; 4 ; 5 ; 6 ; 7 ; 8 ; 9 ; 10 ; 11 ; 12 ; 13 ; 14 ; 15 ; 16 ; 17
# Explain: 0 is sucess ; any other is failure
sub read_config {

    my $file_path = $_[0]; # this is the first function parameter $_[1] is the second etc etc etc
    my $global_config_p = $_[1]; # pointer into the configuration hash
    my $error_flag = 0; # file error flag
    my $error_directory = ""; # to keep the description of which directory failed

    $$global_config_p{"configuration_file"} = $file_path; # path into the configuration file used

    # open file in read mode ( < )
    open(config_file_handle, "<$file_path") or $error_flag = 1;
    if ( $error_flag == 0 ) {
        # read the entire config file line by line
        while(<config_file_handle>) {
            # enter only if there is the = char on the set and the char # is NOT on the begining of the line
            if ( $_ =~ /=/ and not $_ =~ /^#/ ) {

                my ( $config_key, $config_value ) = split ( /=/, $_); # split the line into the configuration key and the configuration value
                $config_key =~ s/^\s+//;  # Delete leading whitespace ^ is the first caracter  \s is space then // replace with nothing if there was /a/ then replace with a
                $config_key =~ s/\s+$//;  # Delete trailing whitespace same thing kind of but $ represents end of line.                
                $config_value =~ s/^\s+//;  # Delete leading whitespace ^ is the first caracter  \s is space then // replace with nothing if there was /a/ then replace with a
                $config_value =~ s/\s+$//;  # Delete trailing whitespace same thing kind of but $ represents end of line
                # use a switch / case this is not a real perl function but it works
                # this will be used to load the global configuration hash
               switch: {
                    /host_os/   && do {
                                        $$global_config_p{"host_os"} = $config_value;
                                    };
                    /backup_type/   && do {
                                        $$global_config_p{"backup_type"} = $config_value;
                                     };
                    /backup_compression_ratio/   && do {
                                        $$global_config_p{"backup_compression_ratio"} = $config_value;
                                     };
                    /input_backup_dir/   && do {
                                        $$global_config_p{"input_backup_dir"} = $config_value;
                                    };
                    /output_backup_dir/   && do {
                                        $$global_config_p{"output_backup_dir"} = $config_value;
                                    };
                    /temporary_dir/   && do {
                                        $$global_config_p{"temporary_dir"} = $config_value;
                                    };
                    /keep_last_n_files/   && do {
                                        $$global_config_p{"keep_last_n_files"} = $config_value;
                                    };
                    /run_before_backup/   && do {
                                        $$global_config_p{"run_before_backup"} = $config_value;
                                    };
                    /run_before_arguments/   && do {
                                        $$global_config_p{"run_before_arguments"} = $config_value;
                                        # remove double spaces
                                        $$global_config_p{"run_before_arguments"} =~ s/( )\1/$1/g; # s/ search  /( ) for spaces  \1 first double chars... first double spaces     /$1 replace with first char in this case always a space  /g global to all string
                                    };
                    /run_after_backup/   && do {
                                        $$global_config_p{"run_after_backup"} = $config_value;
                                    };
                    /run_after_arguments/   && do {
                                        $$global_config_p{"run_after_arguments"} = $config_value;
                                        # remove double spaces
                                        $$global_config_p{"run_after_arguments"} =~ s/( )\1/$1/g; # s/ search  /( ) for spaces  \1 first double chars... first double spaces     /$1 replace with first char in this case always a space  /g global to all string
                                    };
                    /log_file/   && do {
                                        $$global_config_p{"log_file"} = $config_value;
                                    };
                    /ftp_passive_mode/   && do {
                                        $$global_config_p{"ftp_passive_mode"} = $config_value;
                                    };
                    /ftp_timeout/   && do {
                                        $$global_config_p{"ftp_timeout"} = $config_value;
                                    };
                    /ftp_proxy/   && do {
                                        $$global_config_p{"ftp_proxy"} = $config_value;
                                    };
                    /mail_server/   && do {
                                        $$global_config_p{"mail_server"} = $config_value;
                                    };
                                    
                    /mail_port/   && do {
                                        $$global_config_p{"mail_port"} = $config_value;
                                    };
                    /mail_timeout/   && do {
                                        $$global_config_p{"mail_timeout"} = $config_value;
                                    };
                    /mail_from_address/   && do {
                                        $$global_config_p{"mail_from_address"} = $config_value;
                                    };
                    /mail_to_addresses/   && do {
                                        $$global_config_p{"mail_to_addresses"} = $config_value;
                                    };
                    /mail_username/   && do {
                                        $$global_config_p{"mail_username"} = $config_value;
                                    };
                    /mail_password/   && do {
                                        $$global_config_p{"mail_password"} = $config_value;
                                    };
                    /local_db_history_file/   && do {
                                        $$global_config_p{"local_db_history_file"} = $config_value;
                                    };
                }
            }
        }
        # close file
        close (config_file_handle);
    }
    # configure the apropriate backup mode
    my $backup_mode = substr($$global_config_p{"output_backup_dir"},0,6); # extract the first 6 chars... example ftp:// ; scp:// ; etc
    switch: {
            ($backup_mode ne "ftp://" and $backup_mode ne "scp://" ) && do {
                $$global_config_p{"backup_mode"} = "local file system"; # local file system backup mode (ex. /my_directory or d:\my_directory)
             };
            $backup_mode eq "ftp://" && do { #
                $$global_config_p{"backup_mode"} = "ftp"; # ftp backup mode
            };
            $backup_mode eq "scp://" && do { #
                $$global_config_p{"backup_mode"} = "scp"; # scp backup mode
                $error_flag = 17; # this mode is not yet supported
            };
    }

    # test if the file was correctly loaded   used eq and not == because we are using strings
    if ( $error_flag == 0 and ( $$global_config_p{"host_os"} eq "" or $$global_config_p{"backup_type"} eq "" or $$global_config_p{"input_backup_dir"} eq "" or $$global_config_p{"output_backup_dir"} eq "" or $$global_config_p{"temporary_dir"} eq "" or $$global_config_p{"keep_last_n_files"} eq "" or $$global_config_p{"log_file"} eq "" ) ) {
        $error_flag = 2; # mark corrupted file
    }
    # test operating system host values, valid is winnt4 ; win2000/xp ; unix
    if ( $error_flag == 0 and ($$global_config_p{"host_os"} ne "winnt4" and $$global_config_p{"host_os"} ne "win2000/xp" and $$global_config_p{"host_os"} ne "unix") ) {
        $error_flag = 3; # invalid operating system
    }
    # test backup type is valid
    if ( $error_flag == 0 and ($$global_config_p{"backup_type"} ne "rar" and $$global_config_p{"backup_type"} ne "zip" and $$global_config_p{"backup_type"} ne "tar.Z" and $$global_config_p{"backup_type"} ne "tar.bz2" and $$global_config_p{"backup_type"} ne "tar.gz" and $$global_config_p{"backup_type"} ne "tar") ) {
        $error_flag = 4; # invalid backup type
    }
    $$global_config_p{"backup_real_ratio"} = 0; # default no compression
    # test if backup ratio is valid
    if ( $error_flag == 0 and ($$global_config_p{"backup_compression_ratio"} ne "none" and $$global_config_p{"backup_compression_ratio"} ne "minimal" and $$global_config_p{"backup_compression_ratio"} ne "average" and $$global_config_p{"backup_compression_ratio"} ne "maximal" )) {
        $error_flag = 5; # invalid backup ratio
    }
    elsif ( $error_flag == 0 ) { # valid backup ratio set the real value that is going to get passed into the compressor
        switch: {
            $$global_config_p{"backup_compression_ratio"} eq "none" && do { # no compression
                            if ( $$global_config_p{"backup_type"} ne "rar" and $$global_config_p{"backup_type"} ne "zip" ) {
                                $$global_config_p{"backup_real_ratio"} = 1; # almost no compression, none is not avaliable for bzip2, and gzip
                            }
                        };
            $$global_config_p{"backup_compression_ratio"} eq "minimal" && do { # minimal
                            $$global_config_p{"backup_real_ratio"} = 1; # equal to all compressors
                        };
            $$global_config_p{"backup_compression_ratio"} eq "average" && do { # average compression
                            if ( $$global_config_p{"backup_type"} eq "rar" ) {
                                $$global_config_p{"backup_real_ratio"} = 3; # average, just for rar
                            }
                            else {
                                $$global_config_p{"backup_real_ratio"} = 6; # all others use this
                            }
                        };
            $$global_config_p{"backup_compression_ratio"} eq "maximal" && do { # average compression
                            if ( $$global_config_p{"backup_type"} eq "rar" ) {
                                $$global_config_p{"backup_real_ratio"} = 5; # max just for rar
                            }
                            else {
                                $$global_config_p{"backup_real_ratio"} = 9; # all others use this
                            }
                        };
        }
    }

    # check if we are using valid path's ... / for unix host system \ for microsoft systems
    if ( $error_flag == 0 and ( $$global_config_p{"host_os"} eq "unix" and ($$global_config_p{"input_backup_dir"} =~ /\\/ or $$global_config_p{"output_backup_dir"} =~ /\\/ or $$global_config_p{"temporary_dir"} =~ /\\/ or $$global_config_p{"log_file"} =~ /\\/ or $$global_config_p{"local_db_history_file"} =~ /\\/) )) {
        $error_flag = 6; # Using microsoft type of paths \ on a unix system
    }
    # check if we are using valid path's ... / for unix host system \ for microsoft systems
    if ( $error_flag == 0 and ( $$global_config_p{"host_os"} ne "unix" and ( $$global_config_p{"input_backup_dir"} =~ /[\/]/ or ($$global_config_p{"output_backup_dir"} =~ /[\/]/ and $$global_config_p{"backup_mode"} eq "local file system") or $$global_config_p{"temporary_dir"} =~ /[\/]/ or $$global_config_p{"log_file"} =~ /[\/]/ or $$global_config_p{"local_db_history_file"} =~ /[\/]/) )) {
        $error_flag = 7; # Unix unix type of paths / on a microsoft system
    }
    # test if the directory to backup exists
    if ( $error_flag == 0 and !-d $$global_config_p{"input_backup_dir"} ) {
        $error_directory = "Backup directory";
        $error_flag = 8; # mark directory does not exit
    }
    elsif ( $error_flag == 0 and $$global_config_p{"input_backup_dir"} eq "/" ) {
        $error_flag=10; # mark error you cannot backup your root directory
    }

    # test if the backup destiny directory exists
    if ( $error_flag == 0 and !-d $$global_config_p{"output_backup_dir"} and $$global_config_p{"backup_mode"} eq "local file system" ) {
        $error_directory = "Backup destination directory";
        $error_flag = 8; # mark directory does not exit
    }
    # test if the backup temporary directory exists
    if ( $error_flag == 0 and !-d $$global_config_p{"temporary_dir"} ) {
        $error_directory = "Temporary directory";
        $error_flag = 8; # mark directory does not exit
    }

    # check if the value is valid, it must be >= -1  /^[+-]?\d+$/   /^\d+$/
    if (  $error_flag == 0 and ( $$global_config_p{"keep_last_n_files"} !~ /^[+-]?\d+$/ or $$global_config_p{"keep_last_n_files"} < -1 ) ) {
        $error_flag = 9; # # mark directory does not exit
    }

    # test if the backup destiny directory conflits with the temporary or input directory
    if ( $error_flag == 0 and ( $$global_config_p{"input_backup_dir"} eq ($$global_config_p{"temporary_dir"} or $$global_config_p{"output_backup_dir"}) )) {
        $error_directory = "Backup directory";
        $error_flag = 11; # mark directory does not exit
    }
    # if we have arguments defined with no command defined
    if ( $error_flag == 0 and $$global_config_p{"run_before_backup"} eq "" and $$global_config_p{"run_before_arguments"} ne "" ) {
        $error_flag = 12;
    }
    # if we have arguments defined with no command defined
    if ( $error_flag == 0 and $$global_config_p{"run_after_backup"} eq "" and $$global_config_p{"run_after_arguments"} ne "" ) {
        $error_flag = 13;
    }

    if ( $$global_config_p{"backup_mode"} eq "ftp" ) { # if we are using ftp backup mode
        # Check if ftp_passive mode is correcly defined, must be yes/no or empty ""
        if ( $error_flag == 0 and ($$global_config_p{"ftp_passive_mode"} ne "" and $$global_config_p{"ftp_passive_mode"} ne "yes" and $$global_config_p{"ftp_passive_mode"} ne "no") ) {
            $error_flag = 14;
        }
        else { # configure the passive mode parameter (to pass into NET::FTP)
                if ( $$global_config_p{"ftp_passive_mode"} eq "" ) {
                    $$global_config_p{"ftp_passive_mode"}  = "no"; # change to default value
                }
                if ( $$global_config_p{"ftp_passive_mode"} eq "no" ) { # passive mode no
                    $$global_config_p{"ftp_passive_mode_real"} = 0; # The real parameter passed into the NET:FTP, here is [ 0 ; 1 ]
                }
                else { # passive mode yes
                    $$global_config_p{"ftp_passive_mode_real"} = 1; # The real parameter passed into the NET:FTP, here is [ 0 ; 1 ]
                }
        }
        # check if the ftp url is correct, and place all other ftp values in the correct places
        if ( $error_flag == 0 ) {
            my @ftp_array = split(/:/, $$global_config_p{"output_backup_dir"}); # split the url into fields
            my $ftp_array_size = scalar(@ftp_array); # get the array size
            my $ftp_username = "";
            my $ftp_password = "";
            my $ftp_server = "";
            my $ftp_port = "";
            my @ftp_port_servername_ftp_dir = "";
            # general url check: if there is only one @  ; at least 3  / ( ftp:// and the / from the remote ftp dir) and there must be 2 or 3 :
            if ( (($$global_config_p{"output_backup_dir"} =~ tr/@//) != 1) or ($$global_config_p{"output_backup_dir"} =~ tr/\///) < 3 or !(($$global_config_p{"output_backup_dir"} =~ tr/://) >= 2 and ($$global_config_p{"output_backup_dir"} =~ tr/://) <= 3) ) {
                $error_flag = 15;
            }
            else {
                if ( $ftp_array_size ==  3 or $ftp_array_size == 4 ) { # get username, password and ftp server
                    $ftp_username = $ftp_array[1];
                    $ftp_username =~ s/\///g; # remove the // in order to only get the ftp username
                }
                if ( scalar(@ftp_array) == 4 ) { # ftp url with ftp port declared
                    ( $ftp_password, $ftp_server ) = split ( /@/, $ftp_array[2]); # get the ftp password and server name
                    @ftp_port_servername_ftp_dir = split ( /\//, $ftp_array[3]); # split the port and directory paths
                    if ( !($ftp_port_servername_ftp_dir[0] =~ /^\d+$/) ) { # check if port number is a integer number without + or -
                        $error_flag = 15; # mark error
                    }
                    else {
                        $ftp_port = $ftp_port_servername_ftp_dir[0];
                    }
                }
                elsif ( scalar(@ftp_array) == 3 ) { # ftp url without ftp port declared
                    my $ftp_server_ftp_directory = ""; # to countaing ftp_server + directories
                    my $trash = ""; # simple dump
                    ( $ftp_password, $ftp_server_ftp_directory ) = split ( /@/, $ftp_array[2]); # get the ftp password server name + directories
                    ( $ftp_server, $trash ) = split ( /\//, $ftp_server_ftp_directory); # split the servername from the directories
                    $ftp_port = 21; # standard ftp port
                    @ftp_port_servername_ftp_dir = split ( /\//, $ftp_array[2]); # split the directory paths
                }
                else { # URL is incorrect
                    $error_flag = 15; # mark error
                }
            }
            if ( $error_flag == 0 ) {
                # rebuild the correct ftp directory path
                my $ftp_directory = "";
                my $i;
                for ($i = 1; $i < scalar(@ftp_port_servername_ftp_dir); $i++) {
                    $ftp_directory = "$ftp_directory/$ftp_port_servername_ftp_dir[$i]";
                }
                if ( $ftp_directory eq "" ) { # fix in case the user has typed / on the backup directory
                    $ftp_directory = "/";
                }
                # final check
                if ( $ftp_server ne "" and $ftp_port ne "" and $ftp_username ne "" and $ftp_password ne "" and $ftp_directory ne "" ) {
                    # update the main configuration hash
                    $$global_config_p{"ftp_server"} =  $ftp_server;        # Remote ftp server.
                    $$global_config_p{"ftp_server_port"} = $ftp_port;      # Remote ftp server port number.
                    $$global_config_p{"ftp_username"} = $ftp_username;     # FTP Username.
                    $$global_config_p{"ftp_password"} = $ftp_password;     # FTP Password
                    $$global_config_p{"ftp_directory"} = $ftp_directory;   # FTP directory where we will upload the backup file.
                }
                else {
                    $error_flag = 15; # mark error
                }
            }
        }
        if ( $error_flag == 0 ) { # if there was no error get the ftp timeout value
            if ( $$global_config_p{"ftp_timeout"} eq "" ) { # If it's required to put the default Ftp timeout value
                $$global_config_p{"ftp_timeout"} = 480; # default value
            }
            elsif ( !($$global_config_p{"ftp_timeout"} =~ /^\d+$/) or $$global_config_p{"ftp_timeout"} <= 0 ) { # check if the number is a integer
                $error_flag = 16; # mark error
            }
        }
    }

    # Mail stuff
    if ( $$global_config_p{"mail_type"} eq "" ) {
        $$global_config_p{"mail_type"} = "smtp"; # put in the default mail server type
    }
    if ( $error_flag == 0 and $$global_config_p{"mail_type"} ne "smtp" ) {
        $error_flag = 18; # mark error, invalid smtp mail server type
    }
    elsif ( $error_flag == 0 and $$global_config_p{"mail_server"} ne "" ) { # if there was no previous error and a mail server is defined
        # check mail server
        if ( $$global_config_p{"mail_server"} =~ /\s/ ) { # if the mail server name contains a invalid char... \n \r, tabs or spaces
            $error_flag = 19; # mark error, invalid smtp mail server
        }
        # check username + password, if username is used and password is not used or password is used and username not used
        if ( $error_flag == 0 and (($$global_config_p{"mail_username"} ne "" and $$global_config_p{"mail_password"} eq "") or ($$global_config_p{"mail_username"} eq "" and $$global_config_p{"mail_password"} ne "")) ) {
            $error_flag = 20; # mark error, lack of smtp password or lack of smtp username
        }
        # check the from address, must have one @ char and no , char
        if ( $error_flag == 0 and (($$global_config_p{"mail_from_address"} =~ tr/@//) != 1 or ($$global_config_p{"mail_from_address"} =~ tr/,//) != 0) ) {
            $error_flag = 21; # mark error, invalid smtp from address
        }
        # check the to addresses, must have at least has many @ has the , char -1
        # valid is: count(@) - count (,) = 1;
        if ( $error_flag == 0 and (($$global_config_p{"mail_to_addresses"} =~ tr/@//) - ($$global_config_p{"mail_to_addresses"} =~ tr/,//)) != 1 ) {
            $error_flag = 22; # mark error, invalid smtp to addresses
        }
        # check the mail port value
        if ( $$global_config_p{"mail_port"} eq "" ) { # not declared by user on config file
            $$global_config_p{"mail_port"} = 25; # put in the default value
        }
        elsif ( !($$global_config_p{"mail_port"} =~ /^\d+$/) or $$global_config_p{"mail_port"} <= 0 ) { # check if the number is a integer
            $error_flag = 24; # mark error user declared a invalid mail server port
        }
        # check the mail timeout value
        if ( $$global_config_p{"mail_timeout"} eq "" ) { # not declared by user on config file
            $$global_config_p{"mail_timeout"} = 480; # put in the default value
        }
        elsif ( !($$global_config_p{"mail_timeout"} =~ /^\d+$/) or $$global_config_p{"mail_timeout"} <= 0 ) { # check if the number is a integer
            $error_flag = 25; # mark error user declared a invalid mail server port
        }
    }
    elsif ( $error_flag == 0 ) { # no mail server defined
        if ( $$global_config_p{"mail_from_address"} ne "" or $$global_config_p{"mail_to_addresses"} ne "" or $$global_config_p{"mail_username"} ne "" or $$global_config_p{"mail_password"} ne "" or $$global_config_p{"mail_port"} ne "" or $$global_config_p{"mail_timeout"} ne "" ) {
            $error_flag = 23; # mark error, we have some mail configuration but no mail server is declared
        }
    }
    # End of Mail stuff

    if ( $error_flag == 0 ) { # if there was not errors configure the operating system null device, which is used all over the program
        if ( $$global_config_p{host_os} eq "unix" ) { # unix null device
            $$global_config_p{"host_null_device"} = "2>/dev/null";
        }
        else { # microsoft null device
            $$global_config_p{"host_null_device"} = "2>nul";
        }
    }

    # print apropriate messages
    switch: {
        $error_flag == 0 && do { # no error
                            # get the configuration file name, only the name not the full path
                            $$global_config_p{"configuration_file_name"} = get_filedir_name("$$global_config_p{configuration_file}", $global_config_p);
                        };
        $error_flag > 0 && do {
                            print "\nUsing configuration file: [ $file_path ]\n\n";
                        };
        $error_flag == 1 && do {
                            print "Error - opening the configuration file,\ncheck if file exists and if you have read permissions.\n\n";
                        };
        $error_flag == 2 && do {
                            print "Error - configuration file corrupted,\ncheck configuration file.\n\n";
                        };
        $error_flag == 3 && do {
                            print "Error - invalid operating system valid values are: winnt4 ; win2000/xp or unix,\ncheck [ host_os value ].\n\n";
                        };
        $error_flag == 4 && do {
                            print "Error - invalid backup type valid values are: rar ; zip ; tar ; tar.Z ; tar.gz ; tar.bz2,\ncheck [ backup_type value ].\n\n";
                        };
        $error_flag == 5 && do {
                            print "Error - invalid backup compression ratio valid values are: none ; minimal, average ; maximal,\ncheck [ backup_compression_ratio ].\n\n";
                        };
        $error_flag == 6 && do {
                            print "Error - using microsoft type of paths (\\) on a unix system,\ncheck [ host_os ; input_backup_dir ; output_backup_dir ; temporary_dir values ; log_file ; local_db_history_file (if defined) ].\n\n";
                        };
        $error_flag == 7 && do {
                            print "Error - using unix type of paths (/) on a microsoft system,\ncheck [ host_os ; input_backup_dir ; output_backup_dir ; temporary_dir values ; log_file ; local_db_history_file (if defined) ].\n\n";
                        };
        $error_flag == 8 && do {
                            print "Error - [ $error_directory ],\ndoes not exist or you do not have permissions to acess it,\ncheck [ input_backup_dir ; output_backup_dir ; temporary_dir values ].\n\n";
                        };
        $error_flag == 9 && do {
                            print "Error - value is invalid, it must be >= -1,\ncheck [ keep_last_n_files value ].\n\n";
                        };
        $error_flag == 10 && do {
                            print "Error - you cannot use simplebackup to do a root (/) backup of a unix system,\ncheck [ input_backup_dir value ].\n\n";
                        };
        $error_flag == 11 && do {
                            print "Error - [ $error_directory ],\ncannot be the same has the\nbackup destination directory or the temporary directory,\ncheck [ input_backup_dir value ].\n\n";
                        };
        $error_flag == 12 && do {
                            print "Error - you cannot declare arguments without configuring a\nscript/batch file/command to use them,\ncheck [ run_before_backup ; run_before_arguments values ].\n\n";
                        };
        $error_flag == 13 && do {
                            print "Error - you cannot declare arguments without configuring a\nscript/batch file/command to use them,\ncheck [ run_after_arguments ; run_after_arguments values ].\n\n";
                        };
        $error_flag == 14 && do {
                            print "Error - ftp passive mode is badly defined, valid values are: yes/no or not defined at all,\ncheck [ ftp_passive_mode ].\n\n";
                        };
        $error_flag == 15 && do {
                            print "Error - ftp url is badly defined,\ncheck [ output_backup_dir ].\n\n";
                        };
        $error_flag == 16 && do {
                            print "Error - ftp timeout value is invalid,\nit must be a valid number meaning a number over zero, example 480,\ncheck [ ftp_timeout ].\n\n";
                        };
        $error_flag == 17 && do {
                            print "Error - scp (secure copy) backup's are not yet supported,\ncheck [ output_backup_dir ].\n\n";
                        };
        $error_flag == 18 && do {
                            print "Error - invalid mail server type, only smtp mail servers are compatible with simplebackup,\ncheck [ mail_type ].\n\n";
                        };
        $error_flag == 19 && do {
                            print "Error - invalid mail server name or ip,\ncheck [ mail_server ].\n\n";
                        };
        $error_flag == 20 && do {
                            print "Error - incomplete mail authentication, you must fully define the mail username and password or not defining them at all,\ncheck [ mail_username ; mail_password ].\n\n";
                        };
        $error_flag == 21 && do {
                            print "Error - invalid e-mail address,\ncheck [ mail_from_address ].\n\n";
                        };
        $error_flag == 22 && do {
                            print "Error - invalid e-mail address(es),\ncheck [ mail_to_addresses ].\n\n";
                        };
        $error_flag == 23 && do {
                            print "Error - using mail configuration but no mail server is configured,\ncheck [ mail_server ; mail_from_address ; mail_to_addresses ; mail_username ; mail_password ].\n\n";
                        };
        $error_flag == 24 && do {
                            print "Error - mail port value is invalid,\nit must be a valid number meaning a number over zero, example 25,\ncheck [ mail_port ].\n\n";
                        };
        $error_flag == 25 && do {
                            print "Error - mail timeout value is invalid,\nit must be a valid number meaning a number over zero, example 480,\ncheck [ mail_timeout ].\n\n";
                        };
    }

    return $error_flag;
}


#
# Show the configuration read from the conf file,
# NOTE !! this should only be run if the configuration is correct (passed sucessufly in the
# read_config() function!!)
# Will return: 0
# Explain: 0 is sucess
sub show_config {

    my $global_config_p = $_[0]; # configuration pointer

    # put variables in local ones to make them easy to place inside strings
    my $configuration_file = $$global_config_p{"configuration_file"};
    my $configuration_file_name = $$global_config_p{"configuration_file_name"};
    my $host_os = $$global_config_p{"host_os"};
    my $host_null_device = $$global_config_p{"host_null_device"};
    my $backup_type = $$global_config_p{"backup_type"};
    my $backup_compression_ratio = $$global_config_p{"backup_compression_ratio"};
    my $backup_real_ratio = $$global_config_p{"backup_real_ratio"};
    my $input_backup_dir = $$global_config_p{"input_backup_dir"};
    my $output_backup_dir = $$global_config_p{"output_backup_dir"};
    my $temporary_dir = $$global_config_p{"temporary_dir"};
    my $keep_last_n_files = $$global_config_p{"keep_last_n_files"};
    my $run_before_backup = $$global_config_p{"run_before_backup"};
    my $run_before_arguments = $$global_config_p{"run_before_arguments"};
    my $run_after_backup = $$global_config_p{"run_after_backup"};
    my $run_after_arguments = $$global_config_p{"run_after_arguments"};
    my $log_file = $$global_config_p{"log_file"};
    my $local_db_history_file = $$global_config_p{"local_db_history_file"};
    my $explain_keep_last_n_files = "";
    my $ftp_server = "";
    my $ftp_port = "";
    my $ftp_username = "";
    my $ftp_password = "";
    my $ftp_directory = "";
    my $ftp_passive_mode = "";
    my $ftp_proxy = "";
    my $ftp_timeout = "";
    my $mail_server = "";
    my $mail_type = "";
    my $mail_port = "";
    my $mail_timeout = "";
    my $mail_from_address = "";
    my $mail_to_addresses = "";
    my $mail_username = "";
    my $mail_password = "";
    if ( $$global_config_p{"backup_mode"} eq "ftp" ) { # if the backup type is ftp
        $ftp_server = $$global_config_p{"ftp_server"};        # Remote ftp server.
        $ftp_port = $$global_config_p{"ftp_server_port"};      # Remote ftp server port number.
        $ftp_username = $$global_config_p{"ftp_username"};     # FTP Username.
        $ftp_password = $$global_config_p{"ftp_password"};     # FTP Password
        $ftp_directory = $$global_config_p{"ftp_directory"};   # FTP directory where we will upload the backup file.
        $ftp_passive_mode = $$global_config_p{"ftp_passive_mode"}; # FTP passive mode or nor
        $ftp_proxy = $$global_config_p{"ftp_proxy"}; # FTP proxy server (if available)
        if ( $ftp_proxy eq "" ) {
            $ftp_proxy = "None configured";
        }
        $ftp_timeout = $$global_config_p{"ftp_timeout"}; # FTP timeout
    }

    if ( $run_before_backup eq "" ) {
        $run_before_backup = "None configured";
        $run_before_arguments = "None configured";
    }
    if ( $run_after_backup eq "" ) {
        $run_after_backup = "None configured";
        $run_after_arguments = "None configured";
    }

    if ( $keep_last_n_files == -1 ) { 
        $explain_keep_last_n_files = "Keep ALL"; # keep all backup files
    }
    elsif ( $keep_last_n_files == 0 ) {
        $explain_keep_last_n_files = "Keep just the current"; # keep just the current backup file
    }
    else {
        $explain_keep_last_n_files = "$keep_last_n_files plus the current";
    }
    # mail server stuff
    $mail_server = $$global_config_p{"mail_server"};
    if ( $mail_server ne "" ) {
        $mail_port = $$global_config_p{"mail_port"};
        $mail_timeout = $$global_config_p{"mail_timeout"};
        $mail_type = $$global_config_p{"mail_type"};
        $mail_from_address = $$global_config_p{"mail_from_address"};
        $mail_to_addresses = $$global_config_p{"mail_to_addresses"};
        $mail_username = $$global_config_p{"mail_username"};
        $mail_password = $$global_config_p{"mail_password"};
    }
    
    print "\nConfiguration file is [ $configuration_file ],\n\n";
    print "Directories...\n";
    print " Directory to backup (input_backup_dir) : [ $input_backup_dir ]\n";
    print " Backup destiny (output_backup_dir) ....: [ $output_backup_dir ]\n";
    if ( $$global_config_p{"backup_mode"} eq "ftp" ) {
        print "\n  FTP connection details...\n";
        print "   Server ..........: $ftp_server\n";
        print "   Server Port .....: $ftp_port\n";
        print "   Username ........: $ftp_username\n";
        print "   Password ........: $ftp_password\n";
        print "   Remote Directory : $ftp_directory\n";
        print "   Passive mode ....: $ftp_passive_mode\n";
        print "   Proxy ...........: $ftp_proxy\n";
        print "   Timeout .........: $ftp_timeout\n\n";
    }
    print " Temporary backup (temporary_dir) ......: [ $temporary_dir ]\n\n";
    print "Generic...\n";
    print " Selected O.S. (host_os) ....................: $host_os\n";
    print " O.S. null device is ........................: $host_null_device\n";
    print " Backup archiver (backup_type) ..............: $backup_type\n";
    print " Backup ratio (backup_compression_ratio) ....: $backup_compression_ratio\n";
    print " Real backup ratio (passed to archiver) .....: $backup_real_ratio\n";
    print " Backup sessions to keep (keep_last_n_files) : $keep_last_n_files ($explain_keep_last_n_files)\n\n";
    print "External stuff...\n";
    print " Run before backup (run_before_backup) : $run_before_backup\n";
    print "  Arguments (run_before_arguments) ....: $run_before_arguments\n";
    print " Run after backup (run_after_backup) ..: $run_after_backup\n";
    print "  Arguments (run_after_arguments) .....: $run_after_arguments\n\n";
    if ( $local_db_history_file ne "" ) {
        print "Local file database is [ $local_db_history_file ]\n\n";
    }
    print "E-mail reporting...\n";
    if ( $mail_server ne "" ) {
        print "   Mail server type .....: $mail_type\n";
        print "   Mail server ..........: $mail_server\n";
        print "   Mail port ............: $mail_port\n";
        print "   Mail timeout .........: $mail_timeout\n";
        print "   From e-mail address ..: $mail_from_address\n";
        print "   To e-mail address(es) : $mail_to_addresses\n";
        print "   Mail Authentication...\n";
        if ( $mail_username ne "" ) {
            print "      Username ..........: $mail_username\n";
            print "      Password ..........: $mail_password\n";
        }
        else {
            print "      -- None configured --\n";
        }
    }
    else {
        print "   -- None configured --\n";
    }

    print "\nLog file is [ $log_file ]\n\n";

    return 0; # always return sucess
}


#
# Write a log file into the script log file, this should be used to log error messages
# Will also add the message into a message variable define by a pointer
# Will return: 0 ; 1
# Explain: 0 is sucess ; 1 is failure
sub write_file_log {

    my $global_config_p = $_[0]; # configuration pointer
    my $error_msg = $_[1]; # error message
    my $error_flag = 0; # write file control flag
    my $full_date = (localtime(time())); # present date in the default format
    my $configuration_file_name = $$global_config_p{"configuration_file_name"};
    my $new_log = "$full_date - $error_msg\n";
    if  ( defined $_[2] ) { # pointer into a variable that contains all the error messages,
                            # this will be used in the mail backup report, it can be safely ignore
                            # if required
        my $p_all_error_msg = $_[2];
        $$p_all_error_msg = "$$p_all_error_msg$new_log"; # add message into the general variable of all the error messages, the e-mail message will not contain the configuration file name
    }
    
    $new_log = "[ $configuration_file_name ] $new_log"; # add the configuration file name into the file log, this info will not appear in the e-message
    my $log_file = $$global_config_p{"log_file"};
    # open or create the log file in append >> write mode
    open(log_file_handle, ">>$log_file") or $error_flag = 1;
    if ( $error_flag == 0 ) {
        #lock the file for exclusive script use
        flock (log_file_handle, LOCK_EX) or $error_flag = 1;
        # write the log message
        print log_file_handle "$new_log", or $error_flag = 1;
        # trucate the file the data size
        truncate(log_file_handle, tell(log_file_handle)) or $error_flag = 1;
        close (log_file_handle); # close the file
    }
    else {
        print "Warning - unable to write into the log file\n\n";
    }

    return $error_flag; # return the result
}


#
# Work with the local db file which will contain the list of the files managed by simplebackup
# on the current backup session, this is used to work with some more "strange" ftp servers
# can: read and return the entire db in a array (read_add) ; add a new entry (add) ; remove
# one entry (delete).
# Will return: undef ; "" (empty string/array) ; array ; 1
# Explain: 0, "" (empty string/array), array is sucess ; under, 1 is failure
sub work_local_db_history_file {

    my $global_config_p = $_[0]; # configuration pointer
    my $file_name = $_[1]; # file name to be added or removed (it can be empty !)
    my $email_message_body_p = $_[2]; # pointer into the e-mail body string
    my $db_operation = $_[3]; # db operation to be done, valid values are: read_all ; add ; delete

    my $local_db_history_file = $$global_config_p{"local_db_history_file"};
    my $error_flag = 0;
    my @db_array = "";

    if ( -f $local_db_history_file and ($db_operation eq "read_all" or $db_operation eq "delete") ) { # if file exists and it's a read_all or delete operation
        open(local_db_history_file_handle, "<$local_db_history_file") or $error_flag = 1; # open file in read mode
        if ( $error_flag == 0 ) {
            while(<local_db_history_file_handle>) { # read the entire file into the array
                chop($_); # remove the new line aschii chars
                push @db_array, $_;
            }
            close (local_db_history_file_handle); # close file
        }
        else {
            write_file_log($global_config_p, "Error - unable to read the local database file [ $local_db_history_file ]", $email_message_body_p); # write the error message into the script error file
            if ( $db_operation eq "read_all" ) { # return apropriate error code
                return undef; # if there was a error return undefined
            }
        }
    }

    if ( $error_flag == 0 ) {
        # deal with the several options
        switch: {
            $db_operation eq "read_all" && do { # return the entire file in a array
                    if ( scalar(@db_array) > 0 ) { # if data was found on the file
                        return @db_array; # return data
                    }
                    else {
                        return ""; # file was empty, or file does not exist
                    }
                };
            $db_operation eq "add" && do { # add a new entry on the db file
                    # open or create the log file in append >> write mode
                    open(local_db_history_file_handle, ">>$local_db_history_file") or $error_flag = 1;
                    if ( $error_flag == 0 ) {
                        #lock the file for exclusive script use
                        flock (local_db_history_file_handle, LOCK_EX) or $error_flag = 1;
                        # write the log message
                        print local_db_history_file_handle "$file_name\n", or $error_flag = 1;
                        # trucate the file the data size
                        truncate(local_db_history_file_handle, tell(local_db_history_file_handle)) or $error_flag = 1;
                        close (local_db_history_file_handle); # close the file
                    }
                    else {
                        write_file_log($global_config_p, "Error - unable to write into local database file [ $local_db_history_file ], $email_message_body_p"); # write the error message into the script error file
                    }
                };
        $db_operation eq "delete" && do { # return the entire file
                    my $line;
                    my $line_erase_marker = -1; # used the mark the array entry to erase
                    my $counter = 0;
                    # search
                    foreach $line (@db_array)
                    {
                        if ( $line eq "$file_name" ) {
                            $line_erase_marker = $counter;
                            last; # found it now break from the foreach cicle
                        }
                        $counter++;
                    }
                    # erase
                    if ( $line_erase_marker > -1 ) { # erase only if it was found !
                        splice(@db_array, $line_erase_marker, 1); # delete the value from the array
                        # open the file in write mode (clearing the previous contains)
                        open (local_db_history_file_handle, ">$local_db_history_file") or $error_flag = 1;
                        # now write the entire array back into the db file
                        #lock the file for exclusive script use
                        flock (local_db_history_file_handle, LOCK_EX) or $error_flag = 1;
                        foreach $line (@db_array)
                        {
                            print local_db_history_file_handle "$line\n", or $error_flag = 1 if $line ne ""; # print the array, but beware of empty lines
                        }
                        # trucate the file the data size
                        truncate(local_db_history_file_handle, tell(local_db_history_file_handle)) or $error_flag = 1;
                        close (local_db_history_file_handle); # close the file
                    }
                    if ( $error_flag == 1 ) {
                        write_file_log($global_config_p, "Error - unable to write into local database file [ $local_db_history_file ]", $email_message_body_p); # write the error message into the script error file
                    }
                };
        }
    }

    return $error_flag; # return the operation result (only for add ; delete operations)
}


#
# moving files from one directory to another,
# there is a choice to destinct the system from windows NT, Windows2000/XP ; Unix ; ftp servers
# Will return: 0 ; 1 ; 2
# Explain: 0 is sucess ; 1 is failure ; 2 is failure when attempted to erase a damaged backup file
#          from a ftp server
sub move_file {

    # note we used the \" to pass " into the variable to allow usage of names with spaces inside
    my $from_file_system = "\"$_[0]\""; # origin file, to pass to the operating system move/mv
    my $from_file_perl = $_[0]; # origin file, to be passed to perl commands
    my $to_file_system = "\"$_[1]\""; # destiny file, to pass to the operating system move/mv
    my $to_file_perl = $_[1]; # destiny file, to be passed to perl commands
    my $ftp_connection_p = $_[2]; # ftp connection object
    my $global_config_p = $_[3]; # configuration pointer
    my $email_message_body_p = $_[4]; # pointer into the e-mail body string

    my $null_device = $$global_config_p{"host_null_device"}; # operating system null device
    my $error_flag = 0; # to mark errors
    my @mv_command; # will contain the move command and the arguments

    if ( $$global_config_p{backup_mode} eq "local file system" ) { # local file system move (mv / move)
        if ( !-f $to_file_perl ) { # only move files if there is no dupplicated copy on the destiny location
        # deal with Operating System dependend stuff, error messages on windows are redirectry into the virtual file nul on unix into /dev/null
            switch: {
                $$global_config_p{host_os} eq "winnt4" && do {
                                                        @mv_command = ("move", "$from_file_system", "$to_file_system", ">nul"); # here null device must be done this way
                                                    };
                $$global_config_p{host_os} eq "win2000/xp" && do { 
                                                        @mv_command = ("move", "/y", "$from_file_system", "$to_file_system", "$null_device");
                                                    };
                $$global_config_p{host_os} eq "unix" && do { 
                                                        @mv_command = ("mv", "-f", "$from_file_system", "$to_file_system", "$null_device");
                                                    };
            }
            system("@mv_command") == 0 or $error_flag = 1; # if we return 1 something failed
        }
        else { # file with the same name already exists on the destiny directory
            $error_flag = 1; # mark error
        }
    }
    elsif ( $$global_config_p{backup_mode} eq "ftp" ) { # ftp move (NET::FTP) and unlink command
        if ( $ftp_connection_p->size($to_file_perl) ) { # check if the file already exists on the ftp server
            $error_flag = 1; # mark error
        }
        else {
            $ftp_connection_p->binary(); # put in binary download mode
            if ( !$ftp_connection_p->put( "$from_file_perl", "$to_file_perl" ) ) {
                write_file_log($global_config_p, "Error - unable to upload the backup file into the ftp server, it might be lack of permissions, lack of free space or the ftp connection might have dropped, the file is [ $to_file_perl ]", $email_message_body_p); # write the error message into the script error file
                $error_flag = 1; # mark error
                # now attempt to delete the uploaded file from the ftp server, it might have been partly uploaded
                if ( $ftp_connection_p->size($to_file_perl) >= 0 ) { # check if the file exists
                    write_file_log($global_config_p, "Attempting to erase existing incomplete backup file on the ftp server", $email_message_body_p); # write the error message into the script error file
                    if ( !($ftp_connection_p->delete($to_file_perl)) ) { # erase the file
                        # warn that it failed to erase the previous backup file on the ftp server
                        write_file_log($global_config_p, "Error - failed to erase the damaged backup file on the remote ftp server, the connection might have dropped or you might not have the correct permissions", $email_message_body_p); # write the error message into the script error file
                        write_file_log($global_config_p, "Warning - the backup file [ $to_file_perl ] available on the ftp server is corrupted", $email_message_body_p); # write the error message into the script error file
                        return 2; # return error
                    }
                }
             }
            else { # upload sucess, erase the local file
                unlink "$from_file_perl"; # erasing the temporary file
            }
        }
    }
    # check if move worked
    if ( $error_flag == 1 or (-f "$from_file_perl" and (!-f "$to_file_perl" and $$global_config_p{backup_mode} eq "local file system") ) ) { # if there where any errors moving the file show them, being carefull to consider ftp backups
        return 1; # return failure
    }

    return 0; # all ok
}


#
# Receive a full directory path and return the name of the last directory or filename.
# Will return: a string
# Explain: the string it a directory path
sub get_filedir_name {

    my $filedir_name_path = $_[0]; # full path into directory or file
    my $global_config_p = $_[1]; # pointer into the configuration
    my @my_split;

    # deal with Operating System dependend stuff
    if ( $$global_config_p{host_os} eq "unix" ) {
        @my_split = split ( "/", $filedir_name_path ); # unix type paths
    }
    else {
        @my_split = split ( /\\/, $filedir_name_path ); # microsoft type paths
    }

    my $size_value = scalar(@my_split) - 1 ; # read the array size, note array starts at zero size is always bigger
    my $filedir_name = $my_split[$size_value]; # file or directory name

    # deal with Operating System dependend stuff again... check if it's backup up of / or d: ( drive: )
    if ( $$global_config_p{host_os} eq "unix" and ! defined $filedir_name ) { # unix type paths and the path is / (in this case it's detected as undefined)
        $filedir_name = "root"; # backup starting from the root drive /
    }
    elsif ( $filedir_name =~ ":" ) { # microsoft type paths and if it countains the : caracter example D: 
        my @new_my_split = split ( /:/, $filedir_name);
        $filedir_name = "disk_$new_my_split[0]"; # backup of a entire windows partition
    }

    return $filedir_name;
}


#
# Receive a full directory path and return the name of the last directory or filename,
# in a remote server (ftp/scp).
# Will return: a string
# Explain: the string it a directory path
sub get_filedir_name_remote {

    my $filedir_name_path = $_[0]; # full path into directory or file
    my @my_split;

    @my_split = split ( "/", $filedir_name_path ); # unix type paths

    my $size_value = scalar(@my_split) - 1 ; # read the array size, note array starts at zero size is always bigger
    my $filedir_name = $my_split[$size_value]; # file or directory name

    if ( $filedir_name eq "" ) { # if there is no / on the ftp file nameq
        $filedir_name = $filedir_name_path;
    }

    return $filedir_name;
}


#
# Return the previous directory name, example  /directory/another/directory the previous 
# is /directory/another
# Will return the same directory if the path is a root directory example D:\ or /
# Will return: a string
# Explain: the string it a directory path
sub get_previous_filedir_name {

    my $filedir_name_path = $_[0]; # full path into directory or file
    my $global_config_p = $_[1]; # pointer into the configuration
    my @my_split;
    my $file_separator = "";
    my $previous_dir_fullpath;

    # deal with Operating System dependend stuff
    if ( $$global_config_p{host_os} eq "unix" ) {
       @my_split = split ( "/", $filedir_name_path ); # unix type paths
       $file_separator = "/";
    }
    else {
        @my_split = split ( /\\/, $filedir_name_path ); # microsoft type paths
        $file_separator = "\\";
    }
    my $size_value = scalar(@my_split); # array size
    if ( ( $size_value == 0 and $$global_config_p{host_os} eq "unix" ) or ( $size_value == 1 and $$global_config_p{host_os} ne "unix" ) ) { # we are at a starting point... D:\  or / for example
        $previous_dir_fullpath = $filedir_name_path; # unix paths
    }
    $size_value = $size_value - 2; # array size up to the previous backup directory
    if ( $size_value == 0 and $$global_config_p{host_os} ne "unix" ) { # if microsoft path's
        $previous_dir_fullpath = "$my_split[0]\\";
        return $previous_dir_fullpath;
    }

    for my $i ( 0 .. $size_value ) {
        if ( $i == 0 and $size_value > 0 ) { # first part
            $previous_dir_fullpath = "$my_split[$i]";
        }
        else { # next path's
            $previous_dir_fullpath = "$previous_dir_fullpath$file_separator$my_split[$i]";
        }
    }

    return $previous_dir_fullpath;
}


# 
# Get the present backup id session number, to use on the file name... example 5 for the
# full name 5.backup_directory_name.rar
# it will also connect into the ftp server to get the session number, in case ftp backup
# is configured.
# Will return: positive number >= 0
# Explain: 0 is failure , any other is sucess
sub get_present_session_number {

    my $backup_directoryname_ext = $_[0]; # directory name
    my $ftp_connection_p = $_[1]; # pointer into the ftp connection object
    my $global_config_p = $_[2]; # pointer into the configuration
    my $email_message_body_p = $_[3]; # pointer into the e-mail body string

    my $session_number = 0; # default session number
    my $error_flag = 0; # error flag
    my @file_list; # to contain the file list

    if ( $$global_config_p{local_db_history_file} ne "" ) { # if we are using a local db file and not reading the actual file list
        @file_list = work_local_db_history_file( $global_config_p, "", $email_message_body_p, $email_message_body_p, "read_all" ); # read the entire db file
    }

    if ( $$global_config_p{backup_mode} eq "local file system" and $$global_config_p{local_db_history_file} eq "" ) {
        opendir(dir_handle, "$$global_config_p{output_backup_dir}") or $error_flag = 1; # open the load directory
        @file_list = readdir dir_handle; # read the file list
        close(dir_handle);# close the directory handle
    }
    elsif ( $$global_config_p{backup_mode} eq "ftp" and $$global_config_p{local_db_history_file} eq "" ){
        my $ftp_directory = $$global_config_p{ftp_directory}; # prepare to change into the ftp remote directory
        # change into the upload directory
        if ( !$ftp_connection_p->cwd("$ftp_directory") ) {
            write_file_log($global_config_p, "Error - unable to acess the remote ftp directory [ $ftp_directory ]", $email_message_body_p); # write the error message into the script error file
            return 0; # return aborted
        }
        else { # if worked get the file list
            @file_list = $ftp_connection_p->ls;
        }
    }
    if ( $error_flag == 0 ) {
        foreach  my $file_name ( @file_list ) {
            if ( $file_name ne "." and $file_name ne ".." ) { # ignore the system file names . and ..
                if ( $$global_config_p{backup_mode} eq "ftp" and $$global_config_p{local_db_history_file} eq "" ) { # deal with anoying ftp servers that return "strange" names on the ls command, one example of this is the famouse windows war ftp deamon
                    $file_name = get_filedir_name_remote($file_name);
                }
                my $count = 0;
                while ($file_name =~ /$backup_directoryname_ext/g) { $count++ } # trick to only get file names with the correct name, not with the repeted
                my @file_parts = split( /\./, "$file_name" );
                if ( $count == 1 and scalar(@file_parts) >= 3 and $file_parts[0] =~ /^\d+$/ ) { # check if the file name is correct, and the first char is numeric
                    my @file_broken = split( /\./, "$file_name" );
                    if ( $session_number < $file_broken[0] ) { # get the largest session id
                        $session_number = $file_broken[0];
                    }
                }
            }
        }
    }
    else {
        my $bck_directory = $$global_config_p{output_backup_dir};
        write_file_log($global_config_p, "Error - opening the backup destiny directory [ $bck_directory ]", $email_message_body_p); # write the error message into the script error file
        return 0; # return aborted
    }

    return $session_number + 1; # return the net name
}


#
# Clear the previous backup's
# Will return: 0 ; 1
# Explain: 0 is sucess ; 1 failure
sub clear_previous_backups {

    my $session_number = $_[0]; # current session number
    my $backup_directoryname_ext = $_[1]; # part of the file name
    my $ftp_connection_p = $_[2]; # pointer into the ftp connection object
    my $global_config_p = $_[3]; # pointer into the configuration
    my $email_message_body_p = $_[4]; # pointer into the e-mail body string
    my $error_flag = 0; # error flag
    my $erase_file_counter = 0;

    if ( $$global_config_p{keep_last_n_files} == "-1" ) { # nothing to erase
        write_file_log($global_config_p, "No previous backup sessions files where erased, configuration file disables backup sessions erases", $email_message_body_p); # write the error message into the script error file
        return 0;
    }
    else {
        my @file_list; # to contain the file list
        
        if ( $$global_config_p{local_db_history_file} ne "" ) { # if we are using a local db file and not reading the actual file list
            @file_list = work_local_db_history_file( $global_config_p, "", $email_message_body_p, "read_all" ); # read the entire db file
        }

        if ( $$global_config_p{backup_mode} eq "local file system" and $$global_config_p{local_db_history_file} eq "") {
            opendir(dir_handle, "$$global_config_p{output_backup_dir}") or $error_flag = 1; # open the local backup directory
            @file_list = readdir dir_handle; # read the file list
            close(dir_handle);# close the directory handle
        }
        elsif ( $$global_config_p{backup_mode} eq "ftp" and $$global_config_p{local_db_history_file} eq "" ){ # get the list from the ftp server
            my $ftp_directory = $$global_config_p{ftp_directory}; # prepare to change into the ftp remote directory
            # change into the upload directory
            if ( !$ftp_connection_p->cwd("$ftp_directory") ) {
                write_file_log($global_config_p, "Error - unable to acess the remote ftp directory [ $ftp_directory ]", $email_message_body_p); # write the error message into the script error file
                return 0; # return aborted
            }
            else { # if worked get the file list
                @file_list = $ftp_connection_p->ls;
            }
        }
        if ( $error_flag == 0 ) {
             my $check_session_number = $session_number - $$global_config_p{keep_last_n_files}; # calculate the first session to erase
             if ( $check_session_number <= 1 ) {
                 write_file_log($global_config_p, "No previous backup sessions files where erased, no previous backup sessions are in conditions to be erased", $email_message_body_p); # write the error message into the script error file
                 return 0; # there nothing to erase
             }
             foreach  my $file_name ( @file_list ) {
                 if ( $file_name ne "." and $file_name ne ".." ) { # ignore the system file names . and ..
                     if ( $$global_config_p{backup_mode} eq "ftp" and $$global_config_p{local_db_history_file} eq "" ) { # deal with anoying ftp servers that return "strange" names on the ls command, one example of this is the famouse windows war ftp deamon
                        $file_name = get_filedir_name_remote($file_name);
                     }
                     my $count = 0;
                     while ($file_name =~ /$backup_directoryname_ext/g) { $count++ } # trick to only get file names with the correct name, not with the repeted
                     my @file_parts = split( /\./, "$file_name" );
                     if ( $count == 1 and scalar(@file_parts) >= 3 and $file_parts[0] =~ /^\d+$/ ) { # check if the file name is correct, and the first char is numeric
                         my $session_number_id = 0;
                         my $erase_file = "yes";
                         for $session_number_id ( $check_session_number .. $session_number ) { # search if the file is to be erased
                             if ( "$file_name" eq "$session_number_id.$backup_directoryname_ext" ) {
                                 $erase_file = "no"; # file is on the NON ERASE LIST
                             }
                         }
                         if ( $erase_file eq "yes" ) { # file is marked to erase
                             my $erase_me = ""; # to contain the full path to the file that is supose to be erased
                             if ( $$global_config_p{backup_mode} eq "local file system" ) { # local file system backup
                                 $erase_me = $$global_config_p{output_backup_dir};
                             }
                             elsif ( $$global_config_p{backup_mode} eq "ftp" ) { # ftp backup server
                                 $erase_me = $$global_config_p{ftp_directory}; # ftp directory
                             }
                             $erase_me =~ s/[\\\/]$// if "$erase_me" ne "/";;  # Delete trailing \ or / char same thing kind of but $ represents end of line
                             if ( $$global_config_p{host_os} eq "unix" or $$global_config_p{backup_mode} eq "ftp" ) { # unix type paths or ftp server
                                 if ( $erase_me eq "/" ) { # protection against ftp paths that are / (root)
                                     $erase_me = "";
                                 }
                                 $erase_me = "$erase_me/$file_name";
                             }
                             else { # microsoft type paths
                                 $erase_me = "$erase_me\\$file_name";
                             }
                             write_file_log($global_config_p, "Erasing previous backup session file [ $erase_me ]", $email_message_body_p); # write the error message into the script error file
                             # if we are erasing on the local file system
                             if ( $$global_config_p{backup_mode} eq "local file system" ) { # erase the local file
                                 unlink "$erase_me";
                                 $erase_file_counter++; # add counter to erased file
                                 if ( $$global_config_p{local_db_history_file} ne "" and !-f $erase_me ) { # if we are using a local db file and not reading the actual file list
                                     work_local_db_history_file( $global_config_p, "$file_name", $email_message_body_p, "delete" ); # erase the file from the database
                                 }
                             }
                             elsif ($$global_config_p{backup_mode} eq "ftp" ) { # erase the remote ftp file
                                 if ( $ftp_connection_p->size($erase_me) ) { # check if the file exists
                                     if ( $ftp_connection_p->delete($erase_me) ) { # check if the erase of the remote file worked
                                         $erase_file_counter++; # add counter to erased file
                                     }
                                     else { # warn that it failed to erase the previous backup file on the ftp server
                                         write_file_log($global_config_p, "Warning - failed to erase the previous backup file on the remote ftp server, the connection might have dropped or you might not have the correct permissions", $email_message_body_p); # write the error message into the script error file
                                     }
                                 }
                                 else { # file does not exists
                                     write_file_log($global_config_p, "Warning - previous backup file does not exists on the remote ftp server", $email_message_body_p); # write the error message into the script error file
                                 }
                                 if ( $$global_config_p{local_db_history_file} ne "" and !$ftp_connection_p->size($erase_me)) { # if we are using a local db file and the remote file does not exists
                                     work_local_db_history_file( $global_config_p, "$file_name", $email_message_body_p, "delete" ); # erase the file from the database
                                 }
                            }
                         }
                     }
                 }
             }
        }
        else {
            return 1; # something failed
        }
    }
    write_file_log($global_config_p, "Erased [ $erase_file_counter ] previous backup session files", $email_message_body_p); # write the error message into the script error file

    return 0; # all ok
}


#
# Will get all the configuration data from the main configuration hash, plus the 
# dbi connection object and try to connect into the ftp server and login.
# Will return: 0 ; 1
# Explain: 0 is sucess ; 1 failure
sub ftp_connect_login {

    my $ftp_connection_p = $_[0]; # ftp dbi connection pointer
    my $global_config_p = $_[1]; # pointer into the configuration
    my $email_message_body_p = $_[2]; # pointer into the e-mail body string
    
    my $error_flag = 0; # error flag
    my $ftp_server = $$global_config_p{"ftp_server"}; # ftp server
    my $ftp_server_port = $$global_config_p{"ftp_server_port"}; # ftp server port
    my $ftp_passive_mode = $$global_config_p{"ftp_passive_mode"}; # passive mode
    my $ftp_passive_mode_real = $$global_config_p{"ftp_passive_mode_real"};   # The real parameter passed into the NET:FTP, here is 
    my $ftp_proxy = $$global_config_p{"ftp_proxy"};    # FTP Proxy, must work in open mode.
    my $ftp_timeout = $$global_config_p{"ftp_timeout"}; # ftp connection timeout
    my $ftp_username = $$global_config_p{"ftp_username"}; # ftp username
    my $ftp_password = $$global_config_p{"ftp_password"}; # ftp password

    write_file_log($global_config_p, "Connect and login into the ftp server [ $ftp_server:$ftp_server_port ] ; FTP Proxy [  $ftp_proxy ] ; Passive mode [ $ftp_passive_mode ] ; FTP Timeout [ $ftp_timeout ]", $email_message_body_p); # write the error message into the script error file
    if ( $ftp_proxy ne "" ) { # if we have a firewall / proxy server
        $$ftp_connection_p = Net::FTP->new("$ftp_server", Debug => 0, Port => $ftp_server_port, Passive => $ftp_passive_mode_real, Firewall => $ftp_proxy, Timeout => $ftp_timeout );
    }
    else { # if we don't have a firewall / proxy server
        $$ftp_connection_p = Net::FTP->new("$ftp_server", Debug => 0, Port => $ftp_server_port, Passive => $ftp_passive_mode_real, Timeout => $ftp_timeout );
    }
    if ( !$$ftp_connection_p ) { # test if the ftp connection worked
        write_file_log($global_config_p, "Error - unable connect to the ftp server", $email_message_body_p); # write the error message into the script error file
        $error_flag = 1; # return failure
    }

    if ( $error_flag == 0 ) { # if managed to connect, attempt to login
        if ( !$$ftp_connection_p->login("$ftp_username", "$ftp_password") ) { # login into ftp server
            write_file_log($global_config_p, "Error - login into the ftp server [ username: $ftp_username ] ; [ password: ***** ]", $email_message_body_p); # write the error message into the script error file
            $error_flag = 1; # mark error
        }
    }

    return $error_flag; # return result
}


#
# Will return the current date, formated for mail MIME, example : "Wednesday, 5 May, 2004 09:34:37 -0000"
# Will return a date
# Explain: This date is in special format to be send via e-mail and to be understud by e-mail clients
sub get_mail_date {

    # Define the day of the week and the month of the year
    my @days   = ('Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday');
    my @months = ('January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December');

    # Get the current time and format the hour, minutes and seconds.
    my ($sec,$min,$hour,$mday,$mon,$year,$wday) = (localtime(time))[0,1,2,3,4,5,6];
    my $time = sprintf("%02d:%02d:%02d",$hour,$min,$sec);
    $year += 1900; # Add 1900 to the year to get the full 4 digit year

    # Format the date, in this kind "Wednesday, 5 May, 2004 09:34:37 -0000"
    my $date = "$days[$wday], $mday $months[$mon], $year $time -0000";

    # return the date
    return $date;

}


#
# Send a email (if configured)
# Will return 0 ; 1 ; 2 ; 3 ; 4
# Explain: 0 is sucess ; 1 2 3 4 are failures
sub send_email {

    my $my_message_subject = $_[0]; # e-mail message subject
    my $my_message_body = $_[1]; # e-mail message body, the actual email text
    my $x_mailer = $_[2]; # name and version of the program that send the e-mail
    my $global_config_p = $_[3]; # pointer into the configuration

    my $configuration_file_name = $$global_config_p{configuration_file_name};
    my $error_flag = 0; # to mark errors
    my $mail_to_addresses_valid = ""; # this will contain the mail addresses that were accepted by the mail server
    # mail configuration stuff
    my $mail_type = $$global_config_p{"mail_type"};
    my $mail_server = $$global_config_p{"mail_server"};
    my $mail_port = $$global_config_p{"mail_port"};
    my $mail_timeout = $$global_config_p{"mail_timeout"};
    my $mail_from_address = $$global_config_p{"mail_from_address"};
    my $mail_to_addresses = $$global_config_p{"mail_to_addresses"};
    my $mail_username = $$global_config_p{"mail_username"};
    my $mail_password = $$global_config_p{"mail_password"};

    if ( $mail_type eq "smtp" ) {
        my $smtp_connection = ""; # this will countain the SMTP object that represents the connection into the mail server
        # connect into the SMTP mail server
        if ( $mail_username eq "" ) { # connect without authentication 
            $smtp_connection = Net::SMTP->new("$mail_server", Port => "$mail_port", Timeout => "$mail_timeout", Debug => 0 );
        }
        else { # connect with authentication
            $smtp_connection = Net::SMTP->new("$mail_server", Port => "$mail_port", Timeout => "$mail_timeout", Debug => 0, auth("$mail_username", "$mail_password")); # connect without authentication
        }
        if ( !$smtp_connection ) { # check if the SMTP connection is up
            $error_flag = 1; # return failure, fail to connect
        }
        else { # manage to connect, attempt to send e-mail messages
            if ( !$smtp_connection->mail($mail_from_address) ) { # pass the from mail address
                $error_flag = 2; # return failure, invalid from e-mail address
            }
            else { # 
                foreach ( split(',', $mail_to_addresses) ) { # send each e-mail adresss
                    if ( !$smtp_connection->recipient("$_", SkipBad => 1) ) {
                        $mail_to_addresses_valid = "$mail_to_addresses_valid, $_";
                    }
                }
                if ( $mail_to_addresses_valid ne "" ) { # if there is at least one valid email address in the "to" mail section
                    # Prepare the full mail message, including meta data (data, mail program, from, to, subject and body)
                    my $mail_data = "MIME-Version: 1.0" . "\n"
                         . "From: $mail_from_address" . "\n"
                         . "To: " . ( ref($mail_to_addresses_valid) ? join(';', @$mail_to_addresses_valid) : $mail_to_addresses_valid ) . "\n"
                         . "Date: " . get_mail_date() . "\n"
                         . "X-Mailer: " . $x_mailer . "\n"
                         . "Content-Type: text/plain; charset=iso-8859-1" . "\n"
                         . "Subject: " . $my_message_subject . "\n\n"
                         . "Beginning of e-mail report" . "\n\n"
                         . $my_message_body
                         . "End of e-mail report" . "\n\n";
                    if ( !$smtp_connection->data($mail_data) ) { # send the message
                        $error_flag = 4; # return failure, failed to send e-mail message
                    }
                }
                else {
                    $error_flag = 3; # return failure, there was no valid "to" mail address
                }
                $smtp_connection->quit; # exit the smtp mail server
            }
        }
    }

   # write apropriate messages into log file
    switch: {
        $error_flag == 1 && do {
                            if ( $mail_username eq "" ) { # if we are not using authentication
                                write_file_log($global_config_p, "Warning - unable connect to the $mail_type mail server"); # write the error message into the script error file
                            }
                            else { # if we are using authentication
                                write_file_log($global_config_p, "Warning - unable connect to the $mail_type mail server, authentication [ username: $mail_username ; password: ***** ]"); # write the error message into the script error file
                            }
                        };
        $error_flag == 2 && do {
                            write_file_log($global_config_p, "Warning - the \"from\" e-mail address [ $mail_from_address ] is invalid"); # write the error message into the script error file
                        };
        $error_flag == 3 && do {
                            write_file_log($global_config_p, "Warning - there was no valid \"to\" e-mail address, no report was send"); # write the error message into the script error file
                        };
        $error_flag == 4 && do {
                            write_file_log($global_config_p, "Warning - failed to send the e-mail report, the connection with the mail server might have dropped"); # write the error message into the script error file
                        };
    }

    return $error_flag; # return the result

}


#
# Compress backup directory.
# Will return: 0 ; 1
# Explain: 0 is sucess ; 1 failure
sub compress_input_dir {

    my $script_version_int = $_[0]; # script version
    my $last_update_int = $_[1]; # script last update date
    my $global_config_p = $_[2]; # pointer into the configuration
    my $error_flag = 0; # to mark errors
    my @compress_command; # will contain the move command and the arguments
    my $backup_file_temp = ""; # backup file name
    my $null_device = $$global_config_p{"host_null_device"}; # operating system null device
    my $backup_directoryname_ext = ""; # file name of the currect backup file example 15.backup_dirname.rar
    my $backup_directoryname_ext_generic = "";# generic name of the backup files, without the session number, example backup_dirname.rar
    my $session_number = ""; # backup session name
    my $backup_directoryname = ""; # used to build the full directory name
    my $backup_type = ""; # used to build the full directory name
    my $previous_directory = ""; # used to containg the previous directory...
    my $ftp_connection; # to keep the NET:FTP connection object
    my $email_message_body = ""; # will contain all LOG MESSAGES and will be used (if configured)
                                 # to send the full backup log via e-mail
    my $machine_hostname = hostname(); # get the hostname
    my $backup_file_final = ""; # will contain the full path into the destiny backup file
    
    # start logging
    my $config_file = $$global_config_p{configuration_file}; # put it in order to easy put it inside a text string
    my $log_file = $$global_config_p{log_file}; # put it in order to easy put it inside a text string
    write_file_log($global_config_p, "Starting Backup ( simplebackup.pl v$script_version_int - $last_update_int )",\$email_message_body); # write the message into the script error file
    write_file_log($global_config_p, "Hostname is [ $machine_hostname ]", \$email_message_body); # write the message into the script error file
    write_file_log($global_config_p, "Using log file [ $log_file ]", \$email_message_body); # write the message into the script error file
    write_file_log($global_config_p, "Using configuration file [ $config_file ]", \$email_message_body); # write the message into the script error file
    my $backup_mode = $$global_config_p{"backup_mode"};
    write_file_log($global_config_p, "Backup mode is [ $backup_mode ]", \$email_message_body); # write the message into the script error file
    my $host_type = $$global_config_p{host_os};
    write_file_log($global_config_p, "Selected host operating system is [ $host_type ]", \$email_message_body); # write the message into the script error file
    if ( $$global_config_p{"local_db_history_file"} ne "" ) { # if we are using a database file
        my $local_db_history_file = $$global_config_p{"local_db_history_file"};
        write_file_log($global_config_p, "Backup session management will be done using the database file [ $local_db_history_file ]", \$email_message_body); # write the message into the script error file
    }
    if ( $$global_config_p{"backup_mode"} eq "ftp" ) { # connect into the ftp server
        $error_flag = ftp_connect_login(\$ftp_connection, $global_config_p, \$email_message_body);
    }
    if ( $error_flag == 0 ) {
        $backup_directoryname = get_filedir_name("$$global_config_p{input_backup_dir}", $global_config_p); # get the backup directory name
        $backup_type = $$global_config_p{backup_type}; # get the backup type rar, tar, tar.gz etc
        $backup_directoryname_ext = "$backup_directoryname.$backup_type"; # add file extension, tar, rar, etc to build the backup file name ... example somename.rar
        $session_number = get_present_session_number("$backup_directoryname_ext", $ftp_connection, $global_config_p, \$email_message_body); # get the backup session number
        if ( $session_number == 0 ) { # check if there was no error
            write_file_log($global_config_p, "Error - unable to get a new session id number for the backup", \$email_message_body); # write the error message into the script error file
            $error_flag = 1; # return failure
        }
        else { # no error occured continue
            $previous_directory = get_previous_filedir_name("$$global_config_p{input_backup_dir}", $global_config_p); # get the directory path up the previous directory of the one to backup
            if ( ! chdir "$previous_directory" ) { # attempt to change into the previous directory, this is done to build a backup without a full path into the directory, example...   /home/miguel/documents will compress a file with just the directory documents and not the full path
                write_file_log($global_config_p, "Error - unable access the directory [ $previous_directory ] for backup, check permissions",\$email_message_body); # write the error message into the script error file
                $error_flag = 1; # return failure
            }
       }
       if ( $$global_config_p{"backup_mode"} eq "ftp" ) { # connect into the ftp server
           write_file_log($global_config_p, "Disconnecting from ftp server",\$email_message_body); # write the error message into the script error file
           $ftp_connection->quit() if $ftp_connection; # disconnect from ftp server
       }
    }
    if ( $error_flag == 0 ) { # only enter here if there was no previous error
        $backup_directoryname_ext_generic = $backup_directoryname_ext; # keep for latter use this does NOT include the session number on the file name
        $backup_directoryname_ext = "$session_number.$backup_directoryname_ext"; # build the full new compress backup name, example 4.somename.rar

        # deal with Operating System dependend stuff and build the full bath into the backup file
        if ( $$global_config_p{host_os} eq "unix" ) { # unix type paths
            if ( $$global_config_p{temporary_dir} ne "/" ) {
                $backup_file_temp = $$global_config_p{temporary_dir};
            }
            else {
                $backup_file_temp = "";
            }
            $backup_file_temp = "$backup_file_temp/_tmp_$backup_directoryname_ext"; 
        }
        else { # microsoft type paths
            $backup_file_temp = $$global_config_p{temporary_dir};
            $backup_file_temp =~ s/\\$//;  # Delete trailing \ char
            $backup_file_temp = "$backup_file_temp\\_tmp_$backup_directoryname_ext";
        }
        # if there is a available the script/batch file/command declare by run_before_backup  RUN IT !
        if ( $$global_config_p{"run_before_backup"} ne "" ) {
            my @run_before_backup_command;
            push(@run_before_backup_command, $$global_config_p{"run_before_backup"}); # add script/batch file/command to array
            push(@run_before_backup_command, split( / /, $$global_config_p{"run_before_arguments"})); # add arguments to array
            write_file_log($global_config_p, "Executing before backup procedure using command [ @run_before_backup_command ]", \$email_message_body); # write the error message into the script error file
            system("@run_before_backup_command") == 0 or $error_flag = 1; # do backup and if we return 1 something failed
            if ( $error_flag > 0 ) {
                write_file_log($global_config_p, "Error - before backup procedure failed, backup will not continue", \$email_message_body); # write the error message into the script error file
            }
        }

        if ( $error_flag == 0 ) {
            my $compression_ratio = $$global_config_p{"backup_real_ratio"};
            switch: { # prepare backup command
                $$global_config_p{backup_type} eq "rar" && do { # rar type of files
                    @compress_command = ("rar", "a", "-inul", "-isnd", "-m$compression_ratio", "-md4096", "-r", "-s", "-dh", "-rr", "-ow", "\"$backup_file_temp\"", "\"$backup_directoryname\"", "$null_device");                                      
                                               };
                $$global_config_p{backup_type} eq "zip" && do { # tar type of files
                    # deal with Operating System dependend stuff and build the full bath into the backup file
                    if ( $$global_config_p{host_os} eq "unix" ) { # unix operating system (without -S parameter)
                        @compress_command = ("zip", "-r", "-q", "-$compression_ratio", "\"$backup_file_temp\"", "\"$backup_directoryname\"", "$null_device");
                    }
                    else {# windows operating system (with -S parameter, which mean include hidden files)
                        @compress_command = ("zip", "-r", "-q", "-$compression_ratio", "-S", "\"$backup_file_temp\"", "\"$backup_directoryname\"", "$null_device");
                   }
                                               };
                $$global_config_p{backup_type} eq "tar" && do { # tar type of files
                    @compress_command = ("tar", "-cf", "\"$backup_file_temp\"", "\"$backup_directoryname\"", "$null_device");
                                               };
                $$global_config_p{backup_type} eq "tar.Z" && do { # tar + compress type of files
                    @compress_command = ("tar", "-cf", "-", "\"$backup_directoryname\"", "|", "compress", ">", "\"$backup_file_temp\"", "$null_device");
                                               };
                $$global_config_p{backup_type} eq "tar.gz" && do { # tar + gzip type of files
                    @compress_command = ("tar", "-cf", "-", "\"$backup_directoryname\"", "|", "gzip", "-$compression_ratio", ">", "\"$backup_file_temp\"", "$null_device");
                                               };
                $$global_config_p{backup_type} eq "tar.bz2" && do { # tar + bzip2 type of files
                    @compress_command = ("tar", "-cf", "-", "\"$backup_directoryname\"", "|", "bzip2", "-$compression_ratio", ">", "\"$backup_file_temp\"", "$null_device");
                                               };
            }
            write_file_log($global_config_p, "Backup into temporary file using command [ @compress_command ]", \$email_message_body); # write the error message into the script error file
            system("@compress_command") == 0 or $error_flag = 1; # do backup and if we return 1 something failed
            if ( $error_flag > 0 ) {
                write_file_log($global_config_p, "Error - during backup compress process, check permissions, if you have the appropriate operating system commands, if no file (in the directory to be backup) is being locked by a program or operating system, and if there is enough free space on temporary path, indicated by [ temporary_dir ]", \$email_message_body); # write the error message into the script error file
            }
        }
    }
    if ( $error_flag == 0 ) { # if all is ok
        # build final backup, full path and file name
        if ( $$global_config_p{"backup_mode"} eq "local file system" ) { # if the directory is on the local file system
            $backup_file_final = $$global_config_p{output_backup_dir};
        }
        elsif ( $$global_config_p{"backup_mode"} eq "ftp" ) { # if the directory is on the ftp server
            $backup_file_final = $$global_config_p{ftp_directory};
            $error_flag = ftp_connect_login(\$ftp_connection, $global_config_p); # connect into the ftp server
        }
        $backup_file_final =~ s/[\\\/]$// if "$backup_file_final" ne "/";;  # Delete trailing \ or / char same thing kind of but $ represents end of line
        # deal with Operating System dependend stuff and build the full bath into the backup file
        if ( $$global_config_p{host_os} eq "unix" or $$global_config_p{"backup_mode"} eq "ftp" ) { # unix type paths or ftp backup type
            if ( $backup_file_final eq "/" ) { # extra security to deal with / path's
                $backup_file_final = "";
            }
            $backup_file_final = "$backup_file_final/$backup_directoryname_ext";
        }
        else { # microsoft type paths
            $backup_file_final = "$backup_file_final\\$backup_directoryname_ext";
        }
        write_file_log($global_config_p, "Moving temporary backup file [ $backup_file_temp ] into the final backup file [ $backup_file_final ]", \$email_message_body); # write the error message into the script error file
        # move compress backup file from the temporary location into the final backup destination
        my $move_result = move_file("$backup_file_temp", "$backup_file_final", $ftp_connection, $global_config_p, \$email_message_body);
        if ( $move_result != 0 ) {
            $error_flag = 1; # mark error
            write_file_log($global_config_p, "Error - moving temporary backup file [ $backup_file_temp ] into the final backup file [ $backup_file_final ]", \$email_message_body); # write the error message into the script error file
        }
        else { # we only clear previous backups if the moving went ok
            clear_previous_backups($session_number, $backup_directoryname_ext_generic, $ftp_connection, $global_config_p, \$email_message_body);
        }
        if ( $$global_config_p{local_db_history_file} ne "" and ($move_result == 0 or $move_result == 2) ) { # if we are using a local db file and not reading the actual file list, PS: $move_result == 2 means that the upload into the ftp server failed and the partialy uploaded file also failed to be erase
            work_local_db_history_file( $global_config_p, "$backup_directoryname_ext", \$email_message_body, "add" ); # add the file into the db history file
        }
        if ( $$global_config_p{"backup_mode"} eq "ftp" ) { # connect into the ftp server
            write_file_log($global_config_p, "Disconnecting from ftp server", \$email_message_body); # write the error message into the script error file
            $ftp_connection->quit() if $ftp_connection; # disconnect from ftp server
        }
    }

    # if there is a available the script/batch file/command declare by run_after_backup  RUN IT !
    if ( $$global_config_p{"run_after_backup"} ne "" ) {
        my $special_error_flag = 0;
        my @run_after_backup_command;
        push(@run_after_backup_command, $$global_config_p{"run_after_backup"}); # add script/batch file/command to array
        push(@run_after_backup_command, split( / /, $$global_config_p{"run_after_arguments"})); # add arguments to array
        write_file_log($global_config_p, "Executing after backup procedure using command [ @run_after_backup_command ]", \$email_message_body); # write the error message into the script error file
        system(@run_after_backup_command) == 0 or $special_error_flag = 1; # do backup and if we return 1 something failed
        if ( $special_error_flag > 0 ) {
            write_file_log($global_config_p, "Warning - after backup procedure failed", \$email_message_body); # write the error message into the script error file
        }
    }

    if ( -f "$backup_file_temp" ) { # erase the temporary file if one is still available
        write_file_log($global_config_p, "Erasing temporary backup file [ $backup_file_temp ]", \$email_message_body); # write the error message into the script error file
        unlink "$backup_file_temp"; # erase temporary file
    }

    # write the final backup messages and send the e-mail report (if configured by user)
    my $my_configuration_file_name = $$global_config_p{"configuration_file_name"}; # get the configuration file name, something.conf  , not /path/something.conf or c:\path\something.conf
    if ( $error_flag == 0 ) { # write the final message on the log/email report, sucess or failure
        # write the message reporting the backup data file path, $backup_file_final is deal above inside this function, and only if we backup had sucess
        if ( $$global_config_p{"backup_mode"} eq "local file system" ) { # if local file system mode
            write_file_log($global_config_p, "Your backup data is in file [ $backup_file_final ]", \$email_message_body);
        }
        else {
            my $my_backup_mode = $$global_config_p{"backup_mode"}; # backup mode ftp, scp, etc
            my $my_remote_server_name = $$global_config_p{"ftp_server"}; # remote server name, only ftp is supported so far
            my $my_remote_server_port = $$global_config_p{"ftp_server_port"}; # remote server port, only ftp is supported so far
            my $my_remote_username = $$global_config_p{"ftp_username"}; # remote username, only ftp username is supported so far
            write_file_log($global_config_p, "Your backup data is in file [ $backup_file_final ], located in the remote $my_backup_mode server [ $my_remote_server_name:$my_remote_server_port ; username: $my_remote_username ]", \$email_message_body);
        }
        if ( $$global_config_p{"mail_server"} ne "" ) { # if mail server is configured then we can send e-mail read_config() functions tests previoulys
            write_file_log($global_config_p, "Sending e-mail backup report", \$email_message_body); # write the error message into the script error file
            $email_message_body = "$email_message_body" . "End of Backup in ++ sucess ++\n\n";
            send_email("Simplebackup report [ $machine_hostname ; $my_configuration_file_name ; $session_number ]. Backup ended in ++ sucess ++.", $email_message_body, "simplebackup.pl v$script_version_int - $last_update_int", $global_config_p);
        }
        write_file_log($global_config_p, "End of Backup in ++ sucess ++\n\n", \$email_message_body); # write the error message into the script error file
    }
    else {
        if ( $$global_config_p{"mail_server"} ne "" ) { # if mail server is configured then we can send e-mail read_config() functions tests previoulys
            write_file_log($global_config_p, "Sending e-mail backup report", \$email_message_body); # write the error message into the script error file            
            $email_message_body = "$email_message_body" . "End of Backup in -- failure --\n\n";
            send_email("Simplebackup report [ $machine_hostname ; $my_configuration_file_name ; $session_number ]. Backup ended in -- failure --.", $email_message_body, "simplebackup.pl v$script_version_int - $last_update_int", $global_config_p);
        }
        write_file_log($global_config_p, "End of Backup in -- failure --\n\n", \$email_message_body); # write the error message into the script error file
    }
    
    return $error_flag;
}

##########################
# Main
##########################


# test if the commands/parameters are correct
if ( test_commands(@ARGV) != 0 ) {
    show_commands($script_version, $script_last_update, $script_license, $script_authors, $script_email, $script_site); # show the valid commands
    exit 0; # exit from script if the arguments are wrong or there are no arguments
}

# first read the configuration file
if ( read_config($ARGV[1], \%global_config) != 0 ) {
    exit 1; # exit from script if there are any problems on the config file
}

if ( $ARGV[0] eq "--tconf" ) { # show configuration
    show_config(\%global_config);
    print "** Configuration file has passed the tests **\n";

    exit 0; # exit without erros, this is just to show the configuration not to do the backup
}

# Do the backup
if ( compress_input_dir($script_version, $script_last_update, \%global_config) != 0 ) {
    exit 1; # backup failure
};

exit 0; # all worked
